from distutils.log import warn
from os.path import join
from pathlib import Path
import xgboost as xgb
import numpy as np
import pandas as pd
from shaphypetune import BoostBoruta, BoostRFE
from sklearn.base import BaseEstimator

from sklearn.metrics import r2_score, mean_absolute_error
from sklearn import preprocessing

from vickers_hardness.utils.uncertainty import log_cosh_quantile


class VickersHardness(BaseEstimator):
    def __init__(
        self,
        recalibrate=False,
        hyperopt=True,
        xgb_parameters=None,
        result_dir="results",
    ):
        if recalibrate:
            try:
                import uncertainty_toolbox
            except Exception as e:
                # https://blog.finxter.com/how-to-catch-and-print-exception-messages-in-python/
                print(e)
                warn(
                    "Could not import `uncertainty_toolbox`. Maybe try `pip install uncertainty_toolbox`; if already installed, you might be running into issues with Shapely on Windows. See https://github.com/uncertainty-toolbox/uncertainty-toolbox/issues/63. Alternatively, use in `VickersHardness(recalibrate=False)`."
                )
        self.recalibrate = recalibrate
        self.hyperopt = hyperopt
        self.result_dir = result_dir
        if xgb_parameters is None:
            self.xgb_parameters = dict(
                max_depth=4,
                learning_rate=0.05,
                n_estimators=1000,
                verbosity=1,
                booster="gbtree",
                tree_method="auto",
                n_jobs=1,
                gamma=0.0001,
                min_child_weight=8,
                max_delta_step=0,
                subsample=0.6,
                colsample_bytree=0.7,
                colsample_bynode=1,
                reg_alpha=0,
                reg_lambda=4,
                scale_pos_weight=1,
                base_score=0.6,
                num_parallel_tree=1,
                importance_type="gain",
                eval_metric="rmse",
                nthread=4,
            )
        else:
            self.xgb_parameters = xgb_parameters

    def get_params(self, deep=True):
        if not deep:
            return {
                "recalibrate": self.recalibrate,
                "hyperopt": self.hyperopt,
                "result_dir": self.result_dir,
            }
        else:
            return {
                "recalibrate": self.recalibrate,
                "hyperopt": self.hyperopt,
                "result_dir": self.result_dir,
                "xgb_parameters": self.xgb_parameters,
            }

    def fit(self, X_train, y_train, hyperopt=None):
        if hyperopt is None:
            hyperopt = self.hyperopt

        self.X_train = X_train
        self.y_train = y_train
        # %% scale the data
        self.composition_train = self.X_train["formula"].to_frame()
        self.X_train.drop(columns=["formula"], inplace=True)
        self.scaler = preprocessing.StandardScaler().fit(X_train)
        X_train_scl = self.scaler.transform(X_train)

        # %% XGB model construction
        self.xgb_mdl = xgb.XGBRegressor(
            objective="reg:squarederror", **self.xgb_parameters
        )
        if hyperopt:
            self.xgb_mdl = BoostBoruta(self.xgb_mdl, importance_type="shap_importances")
        self.xgb_mdl.fit(X_train_scl, y_train)

        # %% uncertainty quantification
        # https://towardsdatascience.com/confidence-intervals-for-xgboost-cac2955a8fde
        alpha = 0.95
        self.xgb_upper = xgb.XGBRegressor(
            objective=log_cosh_quantile(alpha), **self.xgb_parameters
        )
        self.xgb_lower = xgb.XGBRegressor(
            objective=log_cosh_quantile(1 - alpha), **self.xgb_parameters
        )

        if hyperopt:
            self.xgb_upper = BoostBoruta(
                self.xgb_upper, importance_type="shap_importances"
            )
            self.xgb_lower = BoostBoruta(
                self.xgb_lower, importance_type="shap_importances"
            )

        self.xgb_upper.fit(X_train_scl, y_train)
        self.xgb_lower.fit(X_train_scl, y_train)

    def predict(self, X_test, y_test=None, verbose=True, return_uncertainty=False):
        if y_test is None:
            skip_mae = True
            y_test = np.zeros(X_test.shape[0])
        else:
            skip_mae = False

        # %% Prediction
        self.X_test = X_test
        self.composition_test = self.X_test["formula"].to_frame()
        self.X_test.drop(columns=["formula"], inplace=True)
        X_test_scl = self.scaler.transform(X_test)

        self.y_pred = self.xgb_mdl.predict(X_test_scl)
        self.y_upper = self.xgb_upper.predict(X_test_scl)
        self.y_lower = self.xgb_lower.predict(X_test_scl)

        if isinstance(y_test, pd.Series):
            y_test_vals = y_test.values.ravel()
        elif isinstance(y_test, np.ndarray):
            y_test_vals = y_test.ravel()
        elif isinstance(y_test, list):
            y_test_vals = y_test
        else:
            raise NotImplementedError("y_test should be a Series, ndarray, or list")

        # https://handbook-5-1.cochrane.org/chapter_7/7_7_3_2_obtaining_standard_deviations_from_standard_errors_and.htm
        # hard-coded for 95% CI, sample_size is 1 (?)
        self.y_std = (self.y_upper - self.y_lower) / 3.92
        self.y_std[self.y_std <= 0] = 1e-4
        if self.recalibrate:
            import uncertainty_toolbox as uct

            self.std_recalibrator = uct.recalibration.get_std_recalibrator(
                self.y_pred, self.y_std, y_test_vals, criterion="ma_cal"
            )
            self.y_std_calib = self.std_recalibrator(self.y_std)
        else:
            self.y_std_calib = self.y_std

        if "load" in X_test.columns.tolist():
            self.test_load = X_test["load"]
        else:
            self.test_load = np.nan * np.zeros(X_test.shape[0])
        result_df = pd.DataFrame(
            {
                "actual_hardness": y_test_vals,
                "predicted_hardness": self.y_pred,
                "y_lower": self.y_lower,
                "y_upper": self.y_upper,
                "y_std": self.y_std,
                "y_std_calib": self.y_std_calib,
                "load": self.test_load,
            }
        )
        self.result_df = self.composition_test.join(result_df)

        if verbose and not skip_mae:
            print("MAE: ", mean_absolute_error(y_test, self.y_pred))
            print("R2: ", r2_score(y_test, self.y_pred))

        Path(self.result_dir).mkdir(exist_ok=True)
        self.result_df.to_csv(join(self.result_dir, "predicted_hv.csv"), index=False)

        if return_uncertainty:
            return self.y_pred, self.y_std_calib
        else:
            return self.y_pred




"""Predict hardness values using XGBoost."""
from os.path import join
import xgboost as xgb
import numpy as np
import pandas as pd
from shaphypetune import BoostBoruta, BoostRFE

import matplotlib.pyplot as plt
from plotting import parity_with_err

from sklearn.metrics import r2_score, mean_absolute_error
from sklearn import preprocessing
from sklearn.model_selection import train_test_split

from vickers_hardness.utils.uncertainty import log_cosh_quantile

recalibrate = True

# %% load dataset
X = pd.read_csv("hv_des.csv")
prediction = pd.read_csv("hv_comp_load.csv")
Y = prediction["hardness"]


# %% Train-test split
X_train, X_test, y_train, y_test = train_test_split(
    X, Y, train_size=0.9, test_size=0.1, random_state=100, shuffle=True
)

y_train = y_train.to_frame()
y_test = y_test.to_frame()

# %% scale the data
composition_train = X_train["composition"].to_frame()
composition_test = X_test["composition"].to_frame()
X_train.drop(columns=["composition"], inplace=True)
X_test.drop(columns=["composition"], inplace=True)
scaler = preprocessing.StandardScaler().fit(X_train)
X_train_scl = scaler.transform(X_train)
X_test_scl = scaler.transform(X_test)


# %% XGB model construction
parameters = dict(
    max_depth=4,
    learning_rate=0.05,
    n_estimators=1000,
    verbosity=1,
    booster="gbtree",
    tree_method="auto",
    n_jobs=1,
    gamma=0.0001,
    min_child_weight=8,
    max_delta_step=0,
    subsample=0.6,
    colsample_bytree=0.7,
    colsample_bynode=1,
    reg_alpha=0,
    reg_lambda=4,
    scale_pos_weight=1,
    base_score=0.6,
    num_parallel_tree=1,
    importance_type="gain",
    eval_metric="rmse",
    nthread=4,
)

xgb_mdl = xgb.XGBRegressor(objective="reg:squarederror", **parameters)
xgb_hyp = BoostBoruta(xgb_mdl, importance_type="shap_importances")
xgb_hyp.fit(X_train_scl, y_train)

# %% uncertainty quantification
# https://towardsdatascience.com/confidence-intervals-for-xgboost-cac2955a8fde
alpha = 0.95
xgb_upper = xgb.XGBRegressor(objective=log_cosh_quantile(alpha), **parameters)
xgb_upp_hyp = BoostBoruta(xgb_upper, importance_type="shap_importances")
xgb_upp_hyp.fit(X_train_scl, y_train)
xgb_lower = xgb.XGBRegressor(objective=log_cosh_quantile(1 - alpha), **parameters)
xgb_low_hyp = BoostBoruta(xgb_lower, importance_type="shap_importances")
xgb_low_hyp.fit(X_train_scl, y_train)


# %% Prediction
y_pred = xgb_hyp.predict(X_test_scl)
y_upper = xgb_upp_hyp.predict(X_test_scl)
y_lower = xgb_low_hyp.predict(X_test_scl)

y_test_vals = y_test.values.ravel()

# https://handbook-5-1.cochrane.org/chapter_7/7_7_3_2_obtaining_standard_deviations_from_standard_errors_and.htm
y_std = (y_upper - y_lower) / 3.92  # hard-coded for 95% CI, sample_size is 1 (?)
y_std_calib = y_std

result_df = pd.DataFrame(
    {
        "actual_hardness": y_test_vals,
        "predicted_hardness": y_pred,
        "y_lower": y_lower,
        "y_upper": y_upper,
        "y_std": y_std,
        "y_std_calib": y_std_calib,
        "load": X_test["load"],
    }
)
result_df = composition_test.join(result_df)
result_df.to_csv("predicted_hv.csv", index=False)

parity_with_err(result_df, error_y="y_upper", error_y_minus="y_lower")
parity_with_err(result_df, error_y="y_std", fname="parity_err")
parity_with_err(result_df)

# _, ax = plt.subplots(1, 1, figsize=(5, 5))
# uct.viz.plot_calibration(y_pred, y_std, y_test_vals, ax=ax)
# ax.set_title("")
# plt.show()
# plt.savefig(join("figures", "pre-cal.png"))

# _, ax2 = plt.subplots(1, 1, figsize=(5, 5))
# uct.viz.plot_calibration(y_pred, y_std_calib, y_test_vals, ax=ax2)
# ax2.set_title("")
# plt.show()
# plt.savefig(join("figures", "post-cal.png"))

print("MAE: ", mean_absolute_error(y_test, y_pred))
print("R2: ", r2_score(y_test, y_pred))
print("A file named predicted_hv.csv has been generated.\nPlease check your folder.")

# %% Code Graveyard



# -*- coding: utf-8 -*-
"""Generate descriptors for XGBoost."""
from os.path import join
import pandas as pd
import numpy as np
from pymatgen.core.composition import Composition


class Vectorize_Formula:
    def __init__(self):
        elem_dict = pd.read_csv(
            join("vickerse_hardness", "data", "elementsnew.csv")
        )  # CHECK NAME OF FILE
        self.element_df = pd.DataFrame(elem_dict)
        self.element_df.set_index("Symbol", inplace=True)
        self.column_names = []
        for string in ["avg", "diff", "max", "min"]:
            for column_name in list(self.element_df.columns.values):
                self.column_names.append(string + "_" + column_name)

    def get_features(self, formula):
        try:
            fractional_composition = Composition(
                formula
            ).fractional_composition.as_dict()
            element_composition = Composition(formula).element_composition.as_dict()
            avg_feature = np.zeros(len(self.element_df.iloc[0]))
            std_feature = np.zeros(len(self.element_df.iloc[0]))
            for key in fractional_composition:
                try:
                    avg_feature += (
                        self.element_df.loc[key].values * fractional_composition[key]
                    )
                    diff_feature = (
                        self.element_df.loc[list(fractional_composition.keys())].max()
                        - self.element_df.loc[list(fractional_composition.keys())].min()
                    )
                except Exception as e:
                    print(
                        "The element:",
                        key,
                        "from formula",
                        formula,
                        "is not currently supported in our database",
                    )
                    return np.array([np.nan] * len(self.element_df.iloc[0]) * 4)
            max_feature = self.element_df.loc[list(fractional_composition.keys())].max()
            min_feature = self.element_df.loc[list(fractional_composition.keys())].min()
            std_feature = self.element_df.loc[list(fractional_composition.keys())].std(
                ddof=0
            )

            features = pd.DataFrame(
                np.concatenate(
                    [
                        avg_feature,
                        diff_feature,
                        np.array(max_feature),
                        np.array(min_feature),
                    ]
                )
            )
            features = np.concatenate(
                [
                    avg_feature,
                    diff_feature,
                    np.array(max_feature),
                    np.array(min_feature),
                ]
            )
            return features.transpose()
        except ValueError:
            print(
                "There was an error with the Formula: "
                + formula
                + ", this is a general exception with an unkown error"
            )
            return [np.nan] * len(self.element_df.iloc[0]) * 4


gf = Vectorize_Formula()

# empty list for storage of features
features = []

df = pd.read_csv(join("vickers_hardness", "data", "hv_comp_load.csv"))
# add values to list using for loop
for formula in df["composition"]:
    features.append(gf.get_features(formula))

# feature vectors and targets as X and y
X = pd.DataFrame(features, columns=gf.column_names)
composition_load = df[["composition", "load"]]
predicted = composition_load.join(X)
predicted.to_csv(join("vickers_hardness", "data", "hv_des.csv"), index=False)
print("A file named hv_des.csv has been generated.\nPlease check your folder.")



"""Load the MPDS Vickers hardness dataset."""
from os.path import join
import pandas as pd

from sklearn.metrics import mean_squared_error, mean_absolute_error

from composition_based_feature_vector.composition import generate_features
from sklearn.model_selection import KFold, cross_validate, GroupKFold

from vickers_hardness.utils.plotting import parity_with_err
from vickers_hardness.vickers_hardness_ import VickersHardness

mapper = {"pretty_formula": "formula", "vickers-hardness": "target"}
fpath = join("vickers_hardness", "data", "mpds-vickers-hardness.csv")
data = pd.read_csv(fpath).rename(columns=mapper)
df = data[["formula", "target"]]
df = df.dropna()
df = df.groupby(by="formula", as_index=False).mean()
X, y, formulae, skipped = generate_features(df)

# add formula back in for VickersHardness()
X["formula"] = formulae

hyperopt = True
recalibrate = True
split_by_groups = False  # doesn't do anything since repeat formulae skipped

# %% K-fold cross-validation
if split_by_groups:
    cv = GroupKFold()
    cvtype = "gcv"
    groups=formulae
    # groups = X["load"]
else:
    cv = KFold(shuffle=True, random_state=100)  # ignores groups
    cvtype = "cv"

figure_dir = join("figures", "mpds", f"{cvtype}_hyper{hyperopt}")
result_dir = join("results", "mpds", f"{cvtype}_hyper{hyperopt}")

results = cross_validate(
    VickersHardness(hyperopt=hyperopt, recalibrate=recalibrate, result_dir=result_dir),
    X,
    y,
    groups=groups,
    cv=cv,
    scoring="neg_mean_absolute_error",
    return_estimator=True,
)

estimators = results["estimator"]
result_dfs = [estimator.result_df for estimator in estimators]
merge_df = pd.concat(result_dfs)
merge_df["actual_hardness"] = y

parity_with_err(
    merge_df,
    figfolder=figure_dir,
    error_y="y_upper",
    error_y_minus="y_lower",
    fname=f"parity_ci",
    size=None,
)
parity_with_err(
    merge_df, figfolder=figure_dir, error_y="y_std", fname=f"parity_stderr", size=None,
)
parity_with_err(
    merge_df, figfolder=figure_dir, fname=f"parity_stderr_calib", size=None,
)

y_true, y_pred = [merge_df["actual_hardness"], merge_df["predicted_hardness"]]
mae = mean_absolute_error(y_true, y_pred)
rmse = mean_squared_error(y_true, y_pred, squared=False)
print(f"MAE: {mae:.5f}")
print(f"RMSE: {rmse:.5f}")

# CV and GCV equivalent because duplicates are removed first
# GCV-MAE:  (HV)
# GCV-RMSE:  (HV)

merge_df.sort_index().to_csv(join(result_dir, "results.csv"))

1 + 1




from os.path import join
from pathlib import Path
import numpy as np
import plotly.express as px
import plotly.graph_objects as go


def matplotlibify(fig, size=24, width_inches=3.5, height_inches=3.5, dpi=142):
    # make it look more like matplotlib
    # modified from: https://medium.com/swlh/formatting-a-plotly-figure-with-matplotlib-style-fa56ddd97539)
    font_dict = dict(family="Arial", size=size, color="black")

    # app = QApplication(sys.argv)
    # screen = app.screens()[0]
    # dpi = screen.physicalDotsPerInch()
    # app.quit()

    fig.update_layout(
        font=font_dict,
        plot_bgcolor="white",
        width=width_inches * dpi,
        height=height_inches * dpi,
        margin=dict(r=40, t=20, b=10),
    )

    fig.update_yaxes(
        showline=True,  # add line at x=0
        linecolor="black",  # line color
        linewidth=2.4,  # line size
        ticks="inside",  # ticks outside axis
        tickfont=font_dict,  # tick label font
        mirror="allticks",  # add ticks to top/right axes
        tickwidth=2.4,  # tick width
        tickcolor="black",  # tick color
    )

    fig.update_xaxes(
        showline=True,
        showticklabels=True,
        linecolor="black",
        linewidth=2.4,
        ticks="inside",
        tickfont=font_dict,
        mirror="allticks",
        tickwidth=2.4,
        tickcolor="black",
    )
    fig.update(layout_coloraxis_showscale=False)

    width_default_px = fig.layout.width
    targ_dpi = 300
    scale = width_inches / (width_default_px / dpi) * (targ_dpi / dpi)

    return fig, scale


def parity_with_err(
    result_df,
    x="actual_hardness",
    y="predicted_hardness",
    error_y="y_std_calib",
    error_y_minus=None,
    size="load",
    hover_data=["formula"],
    figfolder="figures",
    fname="parity_err_calib",
    auto_open=True,
    **pxscatter_kwargs,
):
    """Scatter plot with error bars.

    Parameters
    ----------
    result_df : DataFrame
        Must contain at minimum `actual_hardness`, `predicted_hardness`, `y_std_calib`,
        `load`, and `composition`, unless custom values are set for `x`, `y`, `error_y`,
        and `hover_data`, respectively. If any custom kwargs are given to `px.scatter`,
        those must be present as well.
    x : str, optional
        x-axis name used to access column from `result_df`, by default "actual_hardness"
    y : str, optional
        y-axis name used to access column from `result_df`, by default "predicted_hardness"
    error_y : str, optional
        error in y values name used to access column from `result_df`, by default "y_std_calib"
    size : str, optional
        name of variable from `result_df` that will control size of markers, by default "load"
    hover_data : list, optional
        list of names of additional data to include when point is hovered on by mouse, by default ["formula"]
    figfolder : str, optional
        [description], by default "figures"
    fname : str, optional
        [description], by default "parity_err"

    Returns
    -------
    [type]
        [description]
    """
    figpath = join(figfolder, fname)
    Path(figfolder).mkdir(exist_ok=True, parents=True)
    fig = px.scatter(
        result_df,
        x=x,
        y=y,
        error_y=error_y,
        error_y_minus=None,
        size=size,
        hover_data=hover_data,
        **pxscatter_kwargs,
    )

    fig.update_traces(marker=dict(color="red"))

    # parity line
    mx = np.nanmax([result_df["predicted_hardness"], result_df["actual_hardness"]])
    mx2 = mx  # max of both
    # mx, mx2 = np.nanmax(proxy), np.nanmax(target) # max of each
    fig.add_trace(
        go.Line(x=[0, mx], y=[0, mx2], name="parity", line=dict(color="black"))
    )

    fig.write_html(figpath + ".html", auto_open=auto_open)

    fig2, scale = matplotlibify(fig, size=48, dpi=300)

    fig2.update_layout(
        legend=dict(
            font=dict(size=32),
            yanchor="top",
            y=0.99,
            xanchor="left",
            x=0.01,
            bgcolor="rgba(0,0,0,0)",
        )
    )

    fig2.write_image(figpath + ".png")

    return fig, fig2




import numpy as np


def log_cosh_quantile(alpha):
    """Log cosh quantile is a regularized quantile loss function.
    Source: # https://towardsdatascience.com/confidence-intervals-for-xgboost-cac2955a8fde
    Parameters
    ----------
    alpha : float
        confidence level (e.g. 95% confidence == 0.95). Ranges between 0.0 and 1.0.
    """

    def _log_cosh_quantile(y_true, y_pred):
        err = y_pred - y_true
        err = np.where(err < 0, alpha * err, (1 - alpha) * err)
        grad = np.tanh(err)
        hess = 1 / np.cosh(err) ** 2
        return grad, hess

    return _log_cosh_quantile



"""Predict hardness values using XGBoost."""
from os.path import join

import numpy as np
import pandas as pd
from sklearn.metrics import mean_absolute_error, mean_squared_error
from sklearn.model_selection import GroupKFold, KFold, cross_validate

from vickers_hardness.utils.plotting import parity_with_err
from vickers_hardness.vickers_hardness_ import VickersHardness

recalibrate = True
split_by_groups = False

# %% load dataset
X = pd.read_csv(join("vickers_hardness", "data", "hv_des.csv")).rename(
    {"composition": "formula"}
)
groups = X["formula"]
prediction = pd.read_csv(join("vickers_hardness", "data", "hv_comp_load.csv"))
y = prediction["hardness"]

# %% K-fold cross-validation
if split_by_groups:
    cv = GroupKFold()
    cvtype = "gcv"
else:
    cv = KFold(shuffle=True, random_state=100)  # ignores groups
    cvtype = "cv"

results = cross_validate(
    VickersHardness(hyperopt=True, recalibrate=recalibrate),
    X,
    y,
    groups=groups,
    cv=cv,
    scoring="neg_mean_absolute_error",
    return_estimator=True,
)

estimators = results["estimator"]
result_dfs = [estimator.result_df for estimator in estimators]
merge_df = pd.concat(result_dfs)
merge_df["actual_hardness"] = y

parity_with_err(
    merge_df, error_y="y_upper", error_y_minus="y_lower", fname=f"parity_ci_{cvtype}"
)
parity_with_err(merge_df, error_y="y_std", fname=f"parity_stderr_{cvtype}")
parity_with_err(merge_df, fname=f"parity_stderr_calib_{cvtype}")

y_true, y_pred = [merge_df["actual_hardness"], merge_df["predicted_hardness"]]
mae = mean_absolute_error(y_true, y_pred)
rmse = mean_squared_error(y_true, y_pred, squared=False)
print(f"MAE: {mae:.5f}")
print(f"RMSE: {rmse:.5f}")

# CV-MAE: 1.67359 (HV)
# GCV-MAE: 2.2957275 (HV)
# increases by 0.62 (HV)

# CV-RMSE: 2.70621 (HV)
# GCV-RMSE: 3.4197476 (HV)
# increases by 0.71 (HV)

merge_df.sort_index().to_csv(join("results", f"{cvtype}-results.csv"))
1 + 1


# %% Code Graveyard



#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['vickers_hardness', 'vickers_hardness.data', 'vickers_hardness.utils']

package_data = \
{'': ['*'], 'vickers_hardness': ['examples/*', 'tests/*']}

setup(name='vickers_hardness',
      version='0.1.9',
      description='Predicting load dependent Vickers hardness based on chemical composition.',
      author='Ziyan Zhang',
      author_email='"Sterling G. Baird" <sterling.baird@utah.edu>',
      url=None,
      packages=packages,
      package_data=package_data,
     )



"""Load the MPDS Vickers hardness dataset."""
from os.path import join
import numpy as np
import pandas as pd

from composition_based_feature_vector.composition import generate_features
from sklearn.preprocessing import MinMaxScaler

data = pd.read_csv(
    join("vickers_hardness", "data", "mpds-vickers-hardness.csv")
).rename(columns={"pretty_formula": "formula", "vickers-hardness": "target"})
df = data[["formula", "target"]]
df = df.dropna()
df = df.groupby(by="formula", as_index=False).mean()
# df = df.drop_duplicates("formula")
X, y, formulae, skipped = generate_features(df)

data2 = pd.read_csv(join("vickers_hardness", "data", "hv_comp_load.csv")).rename(
    columns={"composition": "formula", "hardness": "target"}
)
df2 = data2[["formula", "target"]]
df2 = df2.groupby(by="formula", as_index=False).mean()
# df2 = df2.drop_duplicates("formula")
X2, y2, formulae2, skipped2 = generate_features(df2)

Xcomb = pd.concat((X, X2), axis=0, ignore_index=True)
scaler = MinMaxScaler()
scaler.fit(Xcomb)
Xscl = scaler.transform(Xcomb)
df_scl = pd.DataFrame(np.round(Xscl, decimals=2), columns=X.columns)
n_uniq = df_scl.drop_duplicates().shape[0]
n_dup = df.shape[0] + df2.shape[0] - n_uniq
print(f"Number of shared formulas: {n_dup}")
# Number of shared formulas: 277
# Number of unique formulas (collectively): 783



"""Test VickersHardness class."""
from os.path import join
import pandas as pd
from sklearn.model_selection import train_test_split

from vickers_hardness.vickers_hardness_ import VickersHardness


def test_vickers_hardness():
    # %% load dataset
    X = pd.read_csv(join("vickers_hardness", "data", "hv_des.csv"))
    prediction = pd.read_csv(join("vickers_hardness", "data", "hv_comp_load.csv"))
    y = prediction["hardness"]

    # %% Train-test split
    X_train, X_test, y_train, y_test = train_test_split(
        X, y, train_size=0.9, test_size=0.1, random_state=100, shuffle=True
    )

    vh = VickersHardness(hyperopt=False)
    vh.fit(X_train, y_train)
    vh.predict(X_test, y_test)


if __name__ == "__main__":
    test_vickers_hardness()
